var output;
var currentVersion = 0;
var calls = new Map();
var channels = new Map();

function initWebSocket(url)
{
	output = document.getElementById("demo");
	testWebSocket(url);
}

function testWebSocket(url)
{
	websocket = new WebSocket(url);
	websocket.onopen = function(evt) { onOpen(evt) };
	websocket.onclose = function(evt) { onClose(evt) };
	websocket.onmessage = function(evt) { onMessage(evt) };
	websocket.onerror = function(evt) { onError(evt) };
}

function onOpen(evt)
{
	output.innerHTML = "";
	console.log("CONNECTED");
	writeToScreen("CONNECTED");

  //
  // request for active channel notification
  var notificationRequest = new Object();
  notificationRequest.messageType = "notificationRequest";
  notificationRequest.notificationType = "activeChannelEvent";
  notificationRequest.toggle = true;

  doSend(JSON.stringify(notificationRequest));

  //
  // request for the initial status
  initializeCalls();
}

function onClose(evt)
{
	console.log("DISCONNECTED: " + evt.reason);
	writeToScreen("DISCONNECTED: " + evt.reason);
}

function onMessage(evt)
{
	var notification = JSON.parse(evt.data);
	console.log(notification);
	if(notification.notificationType == "activeChannelEvent")
	{
		var activeChannelEvent = notification.notification;
		processActiveChannelEvent(activeChannelEvent);
	}
}

function onError(evt)
{
	console.log("ERROR: " + evt.data);
	writeToScreen('<span style="color: red;">ERROR:</span> ' + evt.data);
}

function doSend(message)
{
	console.log("SENT: " + message);
	writeToScreen("SENT: " + message);
	websocket.send(message);
}

function writeToScreen(message)
{
	var pre = document.createElement("p");
	pre.style.wordWrap = "break-word";
	pre.innerHTML = message;
	output.appendChild(pre);
}

function processActiveChannelEvent(evt)
{
	var type = evt.updateType;
	var version = evt.version;
	var channel = evt.activeChannel;
	if(currentVersion > 0 && currentVersion < version)
	{
		//reset
	}
	else if(type == "insert")
	{
		insertChannelEvent(channel);	
	}
	else if(type == "remove")
	{
		checkChannelRemove(channel);
	}

	reloadActiveCalls();
}

function insertChannelEvent(newChannel)
{
	//
	// remove channel if it is already in a call
	checkChannelRemove(newChannel);

	//
	// get or create a call
	call = calls.get(newChannel.callId);
	if (call == null)
	{
		call = createNewCall(newChannel);
	}
	//
	// add or update a channel
	call.channels.set(newChannel.channelId, newChannel);
}

function checkChannelRemove(channelToCheck)
{
	//channelToCheck
	// remove channel from the calls that are having this channel, important for group calls
	for(var call of calls.values())
	{
		var channelToRemove = null;
		for(var channel of call.channels.values())
		{
			if(channel.channelId == channelToCheck.channelId)
			{
				channelToRemove = channel;
				break;
			}
		}

		if(channelToRemove != null)
		{
			removeChannel(call, channelToRemove);
			break;
		}
	}
}

function removeChannel(call, channel)
{
	//
	//remove the channel
	call.channels.delete(channel.channelId);

	//
	//if call don't have anymore channels, remove the call
	if(call.channels.size == 0)
	{
		calls.delete(call.callId);
	}
}

function createNewCall(channel)
{
	var call = new Object();
	call.callId = channel.callId;
	call.channels = new Map();

	calls.set(call.callId, call);
	return call;
}

function reloadActiveCalls()
{
	output.innerHTML = "";
	for(var call of calls.values())
	{
		var callComponent = document.createElement("p");
		for(var channel of call.channels.values())
		{
			if(callComponent.innerHTML != "")
			{
				callComponent.innerHTML += " - "
			}
			callComponent.innerHTML += channel.name + " " + getSubText(channel);
		}
		output.appendChild(callComponent);
	}
}

function getSubText(channel)
{
	let subText = "";
	if(channel.organisationName != null && channel.organisationName != "")
	{
		subText = channel.organisationName;
	}
	else if(channel.phoneUser != null && channel.phoneUser != "")
	{
		subText = channel.phoneUser;
	}
	else if(channel.phone != null && channel.phone != "")
	{
		subText = channel.phone;
	}

	if(channel.parkingSlot != null && channel.parkingSlot != "")
	{
		subText = subText + " " + channel.parkingSlot;
	}

	if(subText != "")
	{
		subText = "(" + subText + ")";
	}

	return subText;
}

function initializeCalls()
{
  var xmlhttp;
  if (window.XMLHttpRequest) 
  {
    xmlhttp = new XMLHttpRequest();
  } 
  else 
  {
    // code for older browsers
    xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
  }
  
  xmlhttp.onreadystatechange = function() 
  {
    if (this.readyState == 4 && this.status == 200) 
    {
      var response = JSON.parse(this.responseText);
      for (var i = 0; i < response.data.length; i++)
      {
      	insertChannelEvent(response.data[i].object);
      }
      reloadActiveCalls();
    }
  };

  var url = mainUrl + "/communications/active_channels";
  xmlhttp.open("GET",url, true);
  xmlhttp.setRequestHeader("Content-Type", "application/json");
  xmlhttp.setRequestHeader("Authorization", "xelion " + authentication);
  xmlhttp.send();
}
